// Path
const path = require("path");

// Fs
const fs = require("fs");

// Utils
const {
  log,
  info,
  done,
  warn,
  error,
  clearConsole,
  exit
} = require("@vue/cli-shared-utils");

// Root
const root = process.cwd();

// Conf
const conf = getConf();

// Sniper
const sniper = {
  // Hard Sniper
  transpileDependencies: ["vue-engineering", "vue-cli-context"]
};

/**
 * Path Resolve
 * @param dir {string}
 * ======== ======== ========
 */
function resolve(dir) {
  return path.join(__dirname, dir);
}

/**
 * Foreach
 * @param source {json}
 * @param callback {function}
 * ======== ======== ========
 */
function foreach(source = {}, callback = () => {}) {
  Object.keys(source).map(item => callback(source[item], item));
}

/**
 * Check
 * @param value {any}
 * @param mode {any}
 * ======== ======== ========
 */
function check(value, mode = Object) {
  return value && value.constructor === mode;
}

/**
 * Deep
 * @param source {json}
 * @param callback {function}
 * ======== ======== ========
 */
function deep(source, callback) {
  // Check
  if (check(source)) {
    // Recursion
    foreach(source, value => deep(value, callback));
    // Factory
    source = callback(source);
  }
}

/**
 * Get Configure
 * ======== ======== ========
 */
function getConf(path = `${root}/scaff.config.js`) {
  // If File
  return fs.existsSync(path) ? require(path) : {};
}

/**
 * Get Argv in Process
 * @param name {string}
 * ======== ======== ========
 */
function getArgv(name, cache = {}) {
  // Argv -- Cant Run in Plugign
  // let [node, path, ...argv] = process.argv;

  // Get Argv From Original Command
  let argv = JSON.parse(process.env.npm_config_argv).original.filter(cmd =>
    /^--/.test(cmd)
  );

  // Mapping
  argv.map(
    item => (
      (item = item.replace(/^--/, "")),
      item.replace(
        /^([\w\.\-\:\/]+)\=?(.*)/g,
        ($0, $1, $2) => ((cache[$1] = `"${$2 || true}"`), $0)
      )
    )
  );

  // Has Name in Argvs
  return name ? cache[name] : cache;
}

/**
 * To Stringify
 * @param conf {json}
 * ======== ======== ========
 */
function toStringify(conf = {}) {
  // Machining
  deep(conf, json => {
    for (let i in json) {
      if (json[i].constructor !== Object) {
        json[i] = `'${json[i]}'`;
      }
    }
  });

  // Return
  return conf;
}

/**
 * hasFile
 * @param path {string}
 * ======== ======== ========
 */
function hasFile(path, callback) {
  fs.access(path, fs.constants.F_OK, e => {
    e ? console.log(`${path} is not defined.`) : callback(path);
  });
}

// Action
module.exports = (api, projOptions, rootOptions) => {
  // Merge User Config 2 Project Options
  Object.assign(projOptions, conf, sniper);

  // Inject Scss Variable
  hasFile(`${root}/src/sheet/variables.scss`, path => {
    projOptions.css = {
      loaderOptions: {
        sass: {
          prependData: `
            @import "@/sheet/variables.scss";
          `
        }
      }
    };
  });

  // Use chainWebpack
  api.chainWebpack(webpackConfig => {
    // Configure
    const configure = require(`${root}/configure.json`);
    // Argvs
    const argvs = getArgv();

    // Env Default
    argvs.env = argvs.env || `"${process.env.npm_config_env || "dev"}"`;
    // Plat Default
    argvs.plat = argvs.plat || `"${process.env.VUE_APP_PLATFORM}"`;

    // Injection
    webpackConfig.plugin("define").tap(
      // Definitions
      definitions => (
        // Merge to Env
        Object.assign(definitions[0]["process.env"], {
          ...argvs,
          ...toStringify(configure)
        }),
        // Return
        definitions
      )
    );
  });
};
